// mozileConfiguration.js

/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Karl Guertin <grayrest@grayrest.com>
 *		James A. Overton <james@overton.ca>
 *		Christian Stocker (Bitflux)
 *		Paul Everitt
 *		David Palm
 *		Lars Kiilerich   <lars[at]v1p[dot]dk>
 *
 * ***** END LICENSE BLOCK ***** */

/*******

Mozile Configuration Javascript
Mozile version 0.6.14

Created by James A. Overton
Created on 2004-03-24

Last revised by James A. Overton
Last revised on 2004-06-24

HISTORY:
	Some code in this file was first found in mozilehtmltb.js, while the 
majority was cut out of mozileTB.js and separated into different files.


DESCRIPTION:
	Injects the eDOM code into the current when told to.

*********/


// Mozile Inject
// Load eDOM and related files into the target document, just like mozileChromeLoader.js


var mozile_js_files = new Array();
mozile_js_files = ["eDOM.js", "eDOMXHTML.js", "domlevel3.js", "mozCE.js", "mozIECE.js"];

function mozileInject(mozileInjectTarget) {
	var f = ["mozileConfiguration.js","mozileInject"];
	mozileDebug(f,2,"Injection starting");
	
		// Don't change this!
	var MOZILE_EXTENSION_DIR = "chrome://mozile/content/eDOM/"; // default location of mozile is in an extension
	
		// POST04: if document.documentElement != HTML then ... or no "head" ...
	var head = mozileInjectTarget.document.getElementsByTagName("head")[0];

	if(!head) {
		mozileDebug(f,1,"Injecting into XML");
		head = mozileInjectTarget.document.documentElement;
	}
	
	mozileLoadConfig(mozileInjectTarget, head);
	
	for (var i=0; i < mozile_js_files.length; i++)
	{
		var scr = document.createElementNS("http://www.w3.org/1999/xhtml","script");
		var src = MOZILE_EXTENSION_DIR + mozile_js_files[i];
		scr.setAttribute("id", mozile_js_files[i]);
		scr.setAttribute("src", src);
		scr.setAttribute("language","JavaScript");
		head.insertBefore(scr, head.firstChild);
	}

	// Global flag to let you know Mozile has been loaded from chrome.
	mozileInjectTarget.mozileChromeLoaded = "true";
	mozileDebug(f,2,"Injection successful");

}
function mozileScourCSS(mozileInjectTarget) {
	var f = ["mozileConfiguration.js","mozileScourCSS"];
	mozileDebug(f,3,"Scouring CSS");
	mozileDebug(f,3,"Number of style sheets: "+mozileInjectTarget.document.styleSheets.length);
	var URL = "";
	for(var	i=0; i<mozileInjectTarget.document.styleSheets.length;i++)	{
		try{
			mozileDebug(f,3,"Style sheet "+ i +" has "+mozileInjectTarget.document.styleSheets[i].cssRules.length+" rules");
			for	(var j=0; j<mozileInjectTarget.document.styleSheets[i].cssRules.length;j++) {
				var	sel	= mozileInjectTarget.document.styleSheets[i].cssRules[j];
				if(sel.selectorText=="mozileconfiguration")	{
					URL = sel.style.backgroundImage;
					URL = URL.substring(4,URL.length-1);
					mozileDebug(f,3,"URL found:"+i+":"+j+":"+URL);
					break;
				}
			}
		} catch(e) {
			// styleSheet is inaccessible, don't worry about it
			mozileDebug(f,2,"Problem with style sheet number "+i+"? "+e);
		}
	}
	return URL;
}

// returns an absolute URL computed from a (possibly relative) URL and a starting point (relativeTo)
function mozileAbsoluteURL(URL, relativeTo) {
	var f = ["mozileConfiguration.js","mozileAbsoluteURL"];
	mozileDebug(f,2,"Start with: "+relativeTo+"  "+URL);
	if(URL.search("://")>0) {
		mozileDebug(f,2,"URL contains '://', so assume it's absolute.");
		return URL;
	}
	relativeTo += " "; // make sure it's a string

  // Remove query string part of already absolute part
  var queryPos = relativeTo.lastIndexOf("?");
  if (queryPos > -1)
    relativeTo = relativeTo.substring(0, queryPos);

  // Check if given URL is relative to host root or current document
  if (URL.charAt(0) == "/") {
    // Make URL relative
    URL = URL.substring(1);

    // Find beginning of hostname
    var hostPos = relativeTo.search("://")+3;
    if (hostPos < 0)
      hostPos = 0;

    // Find end of hostname
    hostPos = relativeTo.indexOf("/", hostPos);
    relativeTo = relativeTo.substring(0, hostPos) + "/";
  }

	var last = relativeTo.lastIndexOf("/");
  var dir = relativeTo.substring(0,last+1);
  	// just tacking the relative URL onto the directory of the reference URL seems to work fine: i.e. http://overton.ca/james/mozile/mozile1/../mozileTest.html becomes http://overton.ca/james/mozile/mozile1/mozileTest.html
	mozileDebug(f,2,"Try: "+ dir + URL);
	URL = dir+URL;
	return URL;
}

function mozileLoadConfig (mozileInjectTarget, headnode) {
	var f = ["mozileConfiguration.js","mozileLoadConfig"];
	mozileDebug(f,2,"Load Config starting");
	
		// Important arrays, hold all the configuration info for the page
	mozileInjectTarget.document.MOZILE_RSD_CONFIG =	new	Array();
	mozileInjectTarget.document.MOZILE_CURRENT_CONFIG =	new	Array();
	
	/* Look	for	an RSD declaration and load	configuration if present */

	var	rsdtype	= "application/rsd+xml";
	var	rsdns =	"http://archipelago.phrasewise.com/rsd";
	var	methodXPath	= "//setting[@name='saveMethod']";
	var	contentXPath = "//setting[@name='saveContent']";
	var httpSavePathXPath = "//setting[@name='httpSavePath']";
	var	pasteModeXPath = "//setting[@name='pasteMode']";
	var	mesXPath = "//setting[@name='mes']";
	var	tagCSSXPath	= "//setting[@name='tagCSS']";
	var	rsdhref, xmlDoc;
	
	
		// first check in CSS
	var CSS_URL = mozileScourCSS(mozileInjectTarget);
	if(CSS_URL && CSS_URL!="") {
		mozileDebug(f,2,"Using configuration from CSS file, href="+CSS_URL);
		rsdhref = CSS_URL;
	} 
		// if that fails, check for a <link>
	else {
		mozileDebug(f,2,"Checking for a RSD link...");	
		// Grab	the	head element and look for the RSD link
		// XXX Moz supports	XPath in HTML now, we could	replace	this w/	XPath
		var	links =	headnode.getElementsByTagName("link");
		for	(i=0; i	< links.length;	i++) {
			// Find	out	if the link	element	has	a type attr	for	RSD
			var	l =	links[i];
			if (l.hasAttribute("type") && l.getAttribute("type") ==	rsdtype) {
				rsdhref	= l.getAttribute("href");
				mozileDebug(f,2,"RSD link found, href="+rsdhref);
			}
		}
	}
	
	
		// handles the RSD file
	function callback (e) {
		// The RSD is loaded, let's	extract	data into a	config object
		mozileDebug(f,2,"RSD file loaded");
	
	//alert(printXML(xmlDoc));
		var	evaluator =	new	XPathEvaluator();
		
		// XXX Hmm,	if the rsd.xml has a namespace decl, the XPath breaks
		// XXX Also, we	should take	any	setting	and	put	it in the array
		var	saveContent = evaluator.evaluate(contentXPath, xmlDoc, null, 
					XPathResult.STRING_TYPE, null).stringValue;
		var	saveMethod = evaluator.evaluate(methodXPath, xmlDoc, null, 
					XPathResult.STRING_TYPE, null).stringValue;
		var	httpSavePath = evaluator.evaluate(httpSavePathXPath, xmlDoc, null, 
					XPathResult.STRING_TYPE, null).stringValue;
		var	pasteMode	= evaluator.evaluate(pasteModeXPath, xmlDoc, null, 
					XPathResult.STRING_TYPE, null).stringValue;
		var	mes	= evaluator.evaluate(mesXPath, xmlDoc, null, 
					XPathResult.STRING_TYPE, null).stringValue;
		var	tagCSS = evaluator.evaluate(tagCSSXPath, xmlDoc, null, 
					XPathResult.STRING_TYPE, null).stringValue;
		
	//alert(method+content+mes+tagCSS+httpSavePath);
	
			// Store the values from the RSD file
		mozileInjectTarget.document.MOZILE_RSD_CONFIG["rsdhref"]	= rsdhref;
		mozileInjectTarget.document.MOZILE_RSD_CONFIG["saveContent"] =	saveContent;
		mozileInjectTarget.document.MOZILE_RSD_CONFIG["saveMethod"]	= saveMethod;

			// make sure there are absolute URLs
                httpSavePath = mozileAbsoluteURL(httpSavePath,rsdhref);
		mozileInjectTarget.document.MOZILE_RSD_CONFIG["httpSavePath"] = httpSavePath;
		mozileInjectTarget.document.MOZILE_RSD_CONFIG["pasteMode"] = pasteMode;

		if(mes!="XHTML") mes = mozileAbsoluteURL(mes,rsdhref);
		mozileInjectTarget.document.MOZILE_RSD_CONFIG["mes"] = mes;

		if(tagCSS!="XHTML") tagCSS = mozileAbsoluteURL(tagCSS,rsdhref);
		mozileInjectTarget.document.MOZILE_RSD_CONFIG["tagCSS"]	= tagCSS;

	
			// Store the same RSD values to the MOZILE_CURRENT_CONFIG array
		mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["saved"] = 'true';
		mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["preset"]	 = 'page';
		mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["rsdhref"]	 = rsdhref;
		mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["saveContent"] =	saveContent;
		mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["saveMethod"]	= saveMethod;
                mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["httpSavePath"] = httpSavePath;
		mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["pasteMode"] = pasteMode;
		mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["mes"] = mes;
		mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["tagCSS"]	= tagCSS;
		
		window.status =	"Done";
		mozileDebug(f,2,"Load Config successful");
		
		mozileLoadMES(mozileInjectTarget, mes);
	}

	if (rsdhref && rsdhref!='') {
		window.status =	"Loading RSD at " + rsdhref;
		mozileDebug(f,2,"No configuration information found, defaults will be applied");
	
			// make sure the URL is absolute
		rsdhref=mozileAbsoluteURL(rsdhref, mozileInjectTarget.document.location);
		
		xmlDoc = mozileInjectTarget.document.implementation.createDocument(rsdns, 
									"doc", null);
		// XXX:	If this	URL	is relative, it	might not work,	so detect this
		xmlDoc.load(rsdhref);
		xmlDoc.onload =	callback;
	}
		// default to the built in XHTML Editing Scheme.
	else {
	
			// Store the same default values to the MOZILE_CURRENT_CONFIG array
		mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["saved"] = 'true';
		mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["preset"] = 'user';
		mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["rsdhref"] = '';
		mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["saveContent"] =	'';
		mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["saveMethod"]	= '';
		mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["httpSavePath"] = '';
		mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["pasteMode"] = '';
		mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["mes"] = "XHTML";
		mozileInjectTarget.document.MOZILE_CURRENT_CONFIG["tagCSS"]	= "XHTML";
		
		mozileLoadMES(mozileInjectTarget,"XHTML");
	}

	return;
}





/* Abandoned in favour of the promised Firefox update mechanism 

function mozileCheckForUpdates(mode) {
	var f = ["mozileConfiguration.js","mozileCheckForUpdates"];
	mozileDebug(f,2,"Checking for updates...");

	var mozileUpdateURL = "http://mozile.mozdev.org/current.xml";
	var	mozileUpdateNS =	"http://archipelago.phrasewise.com/rsd"; // fix this
	var	stableXPath	= "//stable";
	var	developmentXPath	= "//development";
	var xmlDoc;
	
		// handles the current.xml file
	function callback (e) {
	
		var	evaluator =	new	XPathEvaluator();
		
		try {
			var mozileStableNode = evaluator.evaluate(stableXPath, xmlDoc, null, 
						XPathResult.ANY_TYPE, null).iterateNext();
			
			var mozileDevelopmentNode = evaluator.evaluate(developmentXPath, xmlDoc, null, 
						XPathResult.ANY_TYPE, null).iterateNext();
				// now decide whether or not to update		
			if(mode=="devel") {
				var developmentVersion = mozileDevelopmentNode.getAttribute("version");
				mozileDebug(f,2,"Update to development. Current:"+mozileVersion+", Development:"+developmentVersion);
				
					// check all cases
					// if the version number is broken, just return
				if(!developmentVersion || developmentVersion=="") {
						mozileDebug(f,2,"There is a problem with the development version:"+ developmentVersion);
						return;
				}
					// if the current version is installed, do nothing
				if(developmentVersion == mozileVersion) {
					mozileDebug(f,2,"You have the current development version installed");
				}
				else {
						// if a newer version is available, prompt the user
					if(developmentVersion > mozileVersion) {
						mozileDebug(f,2,"Preparing to update to development version "+developmentVersion);
					}
						// the version seems to have been rolled back... prompt the user
					else {
						mozileDebug(f,2,"Preparing to rollback to development version "+developmentVersion);
					}
						// open the updates window
					mozileDialogMode="updates";
					window.open("chrome://mozile/content/XUL/settings.xul", "Settings", "chrome");

				}
			}
			else {
				var stableVersion = mozileStableNode.getAttribute("version");
				mozileDebug(f,2,"Update to stable. Current:"+mozileVersion+", Stable:"+stableVersion);
				
					// check all cases
					// if the version number is broken, just return
				if(!stableVersion || stableVersion=="") {
						mozileDebug(f,2,"There is a problem with the stable version:"+ stableVersion);
						return;
				}
					// if the current version is installed, do nothing
				if(stableVersion == mozileVersion) {
					mozileDebug(f,2,"You have the current stable version installed");
				}
				else {
						// if a newer version is available, prompt the user
					if(stableVersion > mozileVersion) {
						mozileDebug(f,2,"Preparing to update to stable version "+stableVersion);
					}
						// the version seems to have been rolled back... prompt the user
					else {
						mozileDebug(f,2,"Preparing to rollback to stable version "+stableVersion);
					}
						// open the updates window
					mozileDialogMode="updates";
					window.open("chrome://mozile/content/XUL/settings.xul", "Settings", "chrome");
				}
			}
			
		
		
		} catch(e) {
			mozileDebug(f,2,"There was a problem with the update file: "+e);
		}
		
		mozileDebug(f,2,"Update information retrieved");
	}
	
		// go get the file		
	xmlDoc = document.implementation.createDocument(mozileUpdateNS, 
								"doc", null);
	xmlDoc.load(mozileUpdateURL);
	xmlDoc.onload =	callback;

	return;
}


function mozileAutoUpdate() {
	var f = ["mozileConfiguration.js","mozileAutoUpdate"];
	
	var autoUpdate = mozilePrefs.getIntPref("mozile.autoUpdateInt");
	mozileDebug(f,2,"AutoUpdate is set to "+autoUpdate);
	switch(autoUpdate) {
		case 1:
			mozileCheckForUpdates("stable");
			break;
			
		case 2:
			mozileCheckForUpdates("devel");
			break;
			
		default:
			mozileDebug(f,2,"Don't automatically update");
			break;
	}
	
	return;
}
*/