/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		James A. Overton <james@overton.ca>
 *      Andreas Schamberger <mozile@andreass.net>
 *		Paul Everitt
 *
 * ***** END LICENSE BLOCK ***** */

var myBrowser = null;
var appCore = null;
var chromePath = 'chrome://mozile/content/docs/';
var webPath = 'http://mozile.mozdev.org/docs/';

function initBrowser()  {
    myBrowser = document.getElementById("browser-content");

    appCore = Components.classes["@mozilla.org/appshell/component/browser/instance;1"]
               .createInstance(Components.interfaces.nsIBrowserInstance);
    appCore.setWebShellWindow(window);

    // Following lines needed for observing Forward and Back buttons
    window.XULBrowserWindow = new nsBrowserStatusHandler();
    // hook up UI through progress listener
    var interfaceRequestor = myBrowser.docShell.QueryInterface(Components.interfaces.nsIInterfaceRequestor);
    var webProgress = interfaceRequestor.getInterface(Components.interfaces.nsIWebProgress);
    webProgress.addProgressListener(window.XULBrowserWindow, Components.interfaces.nsIWebProgress.NOTIFY_ALL);
    // End observing code

    mozileSetVersionByTagName('dialogheader');

    loadPage(chromePath + 'mozileDocs.html');

    top.setInterval("addStylesheet()",100);

    buildNavigation();
}

function getBrowser()  {
    if (!myBrowser)
        myBrowser = document.getElementById("browser-content");
    return myBrowser;
}

function loadPage(uri)  {
    const nsIWebNavigation = Components.interfaces.nsIWebNavigation;
    getBrowser().webNavigation.loadURI(uri, nsIWebNavigation.LOAD_FLAGS_NONE, null, null, null);
}

function goBack()
{
    var webNavigation = getBrowser().webNavigation;
    if (webNavigation.canGoBack)
        webNavigation.goBack();
}

function goForward()
{
    var webNavigation = getBrowser().webNavigation;
    if (webNavigation.canGoForward)
        webNavigation.goForward();
}

function nsBrowserStatusHandler()
{
}

nsBrowserStatusHandler.prototype =
{
  onStateChange : function(aWebProgress, aRequest, aStateFlags, aStatus) {},
  onProgressChange : function(aWebProgress, aRequest, aCurSelfProgress,
                              aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress) {},
  onSecurityChange : function(aWebProgress, aRequest, state) {},
  onLocationChange : function(aWebProgress, aRequest, aLocation)
  {
    UpdateBackForwardButtons();
  },
  QueryInterface : function(aIID)
  {
    if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
      aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
      aIID.equals(Components.interfaces.nsIXULBrowserWindow) ||
      aIID.equals(Components.interfaces.nsISupports))
      return this;
    throw Components.results.NS_NOINTERFACE;
  },
  setJSStatus : function(status) {},
  setJSDefaultStatus : function(status) {},
  setOverLink : function(link) {}
}

function UpdateBackForwardButtons()
{
    var backBroadcaster = document.getElementById("canGoBack");
    var forwardBroadcaster = document.getElementById("canGoForward");
    var webNavigation = getBrowser().webNavigation;

    var backDisabled = (backBroadcaster.getAttribute("disabled") == "true");
    var forwardDisabled = (forwardBroadcaster.getAttribute("disabled") == "true");

    if (backDisabled == webNavigation.canGoBack)
        backBroadcaster.setAttribute("disabled", !backDisabled);
  
    if (forwardDisabled == webNavigation.canGoForward)
        forwardBroadcaster.setAttribute("disabled", !forwardDisabled);
}

function addStylesheet()
{
    var doc = getBrowser().contentDocument;

    /* <link rel="stylesheet" type="text/css" href="mozileDocs.css"/> */
    var css = doc.createElement("link");
    css.setAttribute("rel", "stylesheet");
    css.setAttribute("type", "text/css");
    css.setAttribute("href", chromePath + 'chrome.css');

    var head = doc.getElementsByTagName("head")[0];
    
    head.appendChild(css);
}

function viewOnlineVersion()
{
    var fileName = getBrowser().contentWindow.location.pathname.substring(location.pathname.lastIndexOf('/')+1);
    loadPage(webPath + fileName);
}

function buildNavigation()
{
	var contents = document.getElementById("contents");
	removeChildren(contents);

    var files = new Array("mozileDocs.html",
        "mozileDevelopment.html",
        "mozileContent.html",
        "mozileLocale.html",
        "mozileSkin.html");

	for(var i=0; i < files.length; i++) {
        var activeFile = chromePath + files[i];

    	var xmlDoc = getFile(activeFile);   
    
        var headlines = xmlDoc.getElementsByTagName("h1");
        var globalHead = headlines[1].firstChild.lastChild.nodeValue;
        
        var h1 = document.createElement("description");
        h1.setAttribute("href", activeFile);
        h1.setAttribute("onclick", "loadPage(this.getAttribute(\"href\"));");
        h1.setAttribute("class", "h1");
        var h1Text = document.createTextNode(globalHead);
        h1.appendChild(h1Text);
        
        contents.appendChild(h1);
    
        var headlines = xmlDoc.getElementsByTagName("h2");
        
    	for(var j=0; j<headlines.length; j++) {
    
            var a = headlines[j].firstChild;
            
            var h2 = document.createElement("description");
            h2.setAttribute("href", activeFile + "#" + a.getAttribute("name"));
            h2.setAttribute("onclick", "loadPage(this.getAttribute(\"href\"));");
            h2.setAttribute("class", "h2");
            var h2Text = document.createTextNode("- " + a.lastChild.nodeValue);
            h2.appendChild(h2Text);
    
            contents.appendChild(h2);
    	}
    }
}

function getFile(filename)
{ 
	xmlhttp = new XMLHttpRequest();
	xmlhttp.overrideMimeType("text/xml");
    xmlhttp.open("GET",filename,false);
    xmlhttp.send(null);
    return xmlhttp.responseXML;
}

function removeChildren(node) 
{	
	if(node.hasChildNodes()!=true) {
		return node;
	}
	var children = node.childNodes;	
	for(var i=0; i<children.length; i++) {
		node.removeChild(children[i]);
	}
	return node;
}